==> 0.4.2&file=in_ffsox-2a-0.4.2-src.tar.gz: in_ffsox-2a-0.4.2/in_ffsox/audio.c <==
/*
 * audio.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "plugin.h"

///////////////////////////////////////////////////////////////////////////////
static AVFrame *audio_ar_frame(void *data)
{
#if defined (LOCAL_AR) // {
  return ((audio_t *)data)->frame;
#else // } {
  return NULL;
#endif // }
}

static AVStream *audio_ar_stream(void *data)
{
  return ((audio_t *)data)->st->st;
}

static int audio_ar_read(void *data, AVPacket *pkt)
{
  return queue_read(&((audio_t *)data)->q,pkt);
}

static void audio_ar_got_frame(void *data, ffsox_ar_t *ar, AVFrame *frame)
{
  if (0==ar->count) {
    audio_t *a=(audio_t *)data;
    samples_t *s=&a->s;
    ffi_st_t *ast=a->st;
    double ts=av_frame_get_best_effort_timestamp(frame);
    double time=1000.0*av_q2d(ast->st->time_base)*ts;
    double length=track_get_length(s->t);

    if (time<=length)
      s->time=time;
  }
}

#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
static int audio_ar_stereo(void *data)
{
  return ((audio_t *)data)->dec->ps->audio.force_stereo;
}
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
static int audio_ar_drc_enabled(void *data)
{
  return ((audio_t *)data)->dec->ps->audio.drc.enabled;
}

static double audio_ar_drc_scale(void *data)
{
  return ((audio_t *)data)->dec->ps->audio.drc.scale;
}
#endif // ]
#endif // ]

static ffsox_ar_cbs_t audio_ar_cbs={
  .frame=audio_ar_frame,
  .stream=audio_ar_stream,
  .read=audio_ar_read,
  .got_frame=audio_ar_got_frame,
#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
  .stereo=audio_ar_stereo,
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
  .drc.enabled=audio_ar_drc_enabled,
  .drc.scale=audio_ar_drc_scale,
#endif // ]
#endif // ]
};

///////////////////////////////////////////////////////////////////////////////
audio_t *audio_init(audio_t *a, decoder_t *dec)
{
#if 0 && defined (FFSOX_AUDIO_CUST) // [
#if defined (PLAY) // [
  track_t *t=dec->t;
#else // ] [
  track_t *t=&dec->t;
#endif // ]
  ffi_st_t *ast=t->ffi.ast;
#endif // ]

  DPRINTF("%s\n",__func__);
  a->dec=dec;

#if defined (LOCAL_AR) // {
  if (NULL==(a->frame=av_frame_alloc()))
    goto error;

  avcodec_get_frame_defaults(a->frame);
#endif // }

  if (NULL==queue_init(&a->q,dec,"Audio",dec->ps->audio.queue_size))
    goto error;

  if (NULL==samples_init(&a->s))
    goto error;

  a->chain=NULL;
#if 0 && defined (FFSOX_AUDIO_CUST) // [
DWRITELN("----------------------------------------------------------");
DVWRITELN("ast=%p",ast);
DWRITELN("----------------------------------------------------------");
  ffsox_ar_init(&a->ar,1.0,ast,a,&audio_ar_cbs);
#else // ] [
  ffsox_ar_init(&a->ar,1.0,a,&audio_ar_cbs);
#endif // ]

  return a;
error:
  audio_cleanup(a);

  return NULL;
}

audio_t *audio_cleanup(audio_t *a)
{
  samples_cleanup(&a->s);
  queue_cleanup(&a->q);

#if defined (LOCAL_AR) // {
  if (NULL!=a->frame)
    av_free(a->frame);
#endif // }

  return a;
}

audio_t *audio_realloc(audio_t *a, track_t *t, size_t nsam)
{
  int sox_needed=track_sox_needed(t,&a->in,&a->out);

  a->st=t->ffi.ast;

#if defined (IN_FFSOX_BS1770) // {
  if (NULL==samples_realloc(&a->s,t,nsam,a->out.nch,a->out.bps,a->out.rate))
    goto error;
#else // } {
  if (NULL==samples_realloc(&a->s,t,nsam,a->out.nch,a->out.bps))
    goto error;
#endif // }

  if (sox_needed&&NULL==(a->chain=ffsox_chain(a)))
    goto error;

  return a;
error:
  return NULL;
}

void audio_reset(audio_t *a)
{
  prefs_t *ps=a->dec->ps;

  //DPRINTF("=== %s\n",__func__);
#if defined (LOCAL_AR) // {
  avcodec_get_frame_defaults(a->frame);
#endif // }
  samples_clear(&a->s);
  queue_reset(&a->q,ps->audio.queue_size);
}

audio_t *audio_run(audio_t *a)
{
  a->hThread=CreateThread(
    NULL,   // _In_opt_   LPSECURITY_ATTRIBUTES lpThreadAttributes,
    0,      // _In_       SIZE_T dwStackSize,
    audio_thread,
            // _In_       LPTHREAD_START_ROUTINE lpStartAddress,
    a,      // _In_opt_   LPVOID lpParameter,
    0,      // _In_       DWORD dwCreationFlags,
    NULL    // _Out_opt_  LPDWORD lpThreadId
  );

  return NULL==a->hThread?NULL:a;
}

void audio_wait(audio_t *a)
{
  if (NULL!=a->hThread) {
    WaitForSingleObject(a->hThread,INFINITE);
    CloseHandle(a->hThread);
    a->hThread=NULL;
  }
}

void audio_stop(audio_t *a)
{
  if (NULL!=a->chain) {
    sox_delete_effects_chain(a->chain);
    a->chain=NULL;
  }

  a->st=NULL;
}

void audio_kill(audio_t *a)
{
  samples_kill(&a->s);
}

#if defined (IN_FFSOX_BS1770) // {
void audio_adapt(audio_t *a)
{
  //DPRINTF("=== %s\n",__func__);
  samples_adapt(&a->s,NULL==a->chain?&a->s.q:&a->ar.q);
}
#endif // }

DWORD WINAPI audio_thread(LPVOID lpParam)
{
  audio_t *a=lpParam;
  ffsox_ar_t *ar=&a->ar;

#if defined (IN_FFSOX_BS1770) // {
  ffsox_ar_reset(ar,a->s.q,&a->s.lufs);
#else // } {
  //ffsox_ar_reset(ar,a->s.q);
  ffsox_ar_reset(ar,a->s.q,NULL);
#endif // }

  if (NULL==a->chain) {
    DPUTS("NATIVE ...\n");
    ffsox_ar_next(ar);

    while (ffsox_ar_valid(ar)) {
      samples_copy(&a->s,ar);
      ffsox_ar_next(ar);
    }
  }
  else {
    DPUTS("SOX ...\n");
    sox_flow_effects(a->chain,NULL,NULL);
  }

  // stop samples
  WaitForSingleObject(a->s.hMutex,INFINITE);

  if (STATE_RUN==a->s.state)
    a->s.state=STATE_EOF;

  SetEvent(a->s.hEventRead);
  ReleaseMutex(a->s.hMutex);

  return 0;
}

==> 0.6.0&file=in_ffsox-2a-0.6.0-src.tar.gz: in_ffsox-2a-0.6.0/in_ffsox/audio.c <==
/*
 * audio.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "plugin.h"

#if defined (DEBUG) // [
#if defined (SOX) // [
#define SOX_ERROR_MNEMONIC
#endif // ]

#if defined (SOX_ERROR_MNEMONIC) // [
///////////////////////////////////////////////////////////////////////////////
static const char *sox_error_mnemonic(int sox_error)
{
  switch (sox_error) {
  case SOX_SUCCESS:
    return "SOX_SUCCESS";
  case SOX_EOF:
    return "SOX_EOF";
  case SOX_EHDR:
    return "SOX_EHDR";
  case SOX_EFMT:
    return "SOX_EFMT";
  case SOX_ENOMEM:
    return "SOX_ENOMEM";
  case SOX_EPERM:
    return "SOX_EPERM";
  case SOX_ENOTSUP:
    return "SOX_ENOTSUP";
  case SOX_EINVAL:
    return "SOX_EINVAL";
  default:
    return "undefined";
  }
}
#endif // ]
#endif // ]

#if ! defined (FFSOX_FFI_DECODE) // [
///////////////////////////////////////////////////////////////////////////////
static AVFrame *audio_ar_frame(void *data)
{
#if defined (LOCAL_AR) // {
  return ((audio_t *)data)->frame;
#else // } {
  return NULL;
#endif // }
}

static AVStream *audio_ar_stream(void *data)
{
  return ((audio_t *)data)->st->st;
}

static int audio_ar_read(void *data, AVPacket *pkt)
{
  return queue_read(&((audio_t *)data)->q,pkt);
}

//#define FFSOX_TIME_CMP_LENGTH
//#define FFSOX_JUST_FRAME_ZERO
static void audio_ar_got_frame(void *data, ffsox_ar_t *ar, AVFrame *frame)
{
#if defined (FFSOX_JUST_FRAME_ZERO) // [
  if (0==ar->count) {
#endif // ]
    audio_t *a=(audio_t *)data;
    samples_t *s=&a->s;
    ffi_st_t *ast=a->st;
#if defined (FFSOX_DEPRECATED) // [
    double ts=frame->pts;
    //double ts=frame->best_effort_timestamp;
#else // ] [
    double ts=av_frame_get_best_effort_timestamp(frame);
#endif // ]
    double time=1000.0*av_q2d(ast->st->time_base)*ts;
#if defined (FFSOX_TIME_CMP_LENGTH) // [
    double length=track_get_length(s->t);
#endif // ]

#if defined (FFSOX_TIME_CMP_LENGTH) // [
    if (time<=length)
      s->time=time;
#else // ] [
    s->time=time;
#endif // ]
#if defined (FFSOX_JUST_FRAME_ZERO) // [
  }
#endif // ]
}

#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
static int audio_ar_stereo(void *data)
{
  return ((audio_t *)data)->dec->ps->audio.force_stereo;
}
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
static int audio_ar_drc_enabled(void *data)
{
  return ((audio_t *)data)->dec->ps->audio.drc.enabled;
}

static double audio_ar_drc_scale(void *data)
{
  return ((audio_t *)data)->dec->ps->audio.drc.scale;
}
#endif // ]
#endif // ]

static ffsox_ar_cbs_t audio_ar_cbs={
  .frame=audio_ar_frame,
  .stream=audio_ar_stream,
  .read=audio_ar_read,
  .got_frame=audio_ar_got_frame,
#if defined (FFSOX_AUDIO_CUST) // [
#if defined (FFSOX_AUDIO_STEREO) // [
  .stereo=audio_ar_stereo,
#endif // ]
#if defined (FFSOX_AUDIO_DRC) // [
  .drc.enabled=audio_ar_drc_enabled,
  .drc.scale=audio_ar_drc_scale,
#endif // ]
#endif // ]
};
#endif // ]

///////////////////////////////////////////////////////////////////////////////
audio_t *audio_init(audio_t *a, decoder_t *dec)
{
#if 0 && defined (FFSOX_AUDIO_CUST) // [
  track_t *t=&dec->t;
  ffi_st_t *ast=t->ffi.ast;
#endif // ]

  DMARKLN();
  a->dec=dec;

#if defined (LOCAL_AR) // {
  if (NULL==(a->frame=av_frame_alloc()))
    goto error;

  avcodec_get_frame_defaults(a->frame);
#endif // }

  if (NULL==queue_init(&a->q,dec,"Audio",dec->ps->audio.queue_size))
    goto error;

  if (NULL==samples_init(&a->s))
    goto error;

#if defined (SOX) // [
  a->chain=NULL;
#endif // ]
  ffsox_ar_init(&a->ar,1.0,a,&audio_ar_cbs);

  return a;
error:
  audio_cleanup(a);

  return NULL;
}

audio_t *audio_cleanup(audio_t *a)
{
  samples_cleanup(&a->s);
  queue_cleanup(&a->q);

#if defined (LOCAL_AR) // {
  if (NULL!=a->frame)
    av_free(a->frame);
#endif // }

  return a;
}

audio_t *audio_realloc(audio_t *a, track_t *t, size_t nsam)
{
#if defined (SOX) // [
  int sox_needed=track_sox_needed(t,&a->in,&a->out);
#else // ] [
  track_sox_needed(t,&a->in,&a->out);
#endif // ]

  a->st=t->ffi.ast;

#if defined (IN_FFSOX_BS1770) // {
  if (NULL==samples_realloc(&a->s,t,nsam,a->out.nch,a->out.bps,a->out.rate))
    goto error;
#else // } {
  if (NULL==samples_realloc(&a->s,t,nsam,a->out.nch,a->out.bps))
    goto error;
#endif // }

#if defined (SOX) // [
  if (sox_needed) {
    DWRITELN("allocating sox chain");

    if (!(a->chain=ffsox_chain(a))) {
      DMESSAGE("allocating sox chain");
      goto error;
    }
  }
#endif // ]

  return a;
error:
  return NULL;
}

void audio_reset(audio_t *a)
{
  prefs_t *ps=a->dec->ps;

  //DMARKLN();
#if defined (LOCAL_AR) // {
  avcodec_get_frame_defaults(a->frame);
#endif // }
  samples_clear(&a->s);
  queue_reset(&a->q,ps->audio.queue_size);
}

audio_t *audio_run(audio_t *a)
{
  a->hThread=CreateThread(
    NULL,   // _In_opt_   LPSECURITY_ATTRIBUTES lpThreadAttributes,
    0,      // _In_       SIZE_T dwStackSize,
    audio_thread,
            // _In_       LPTHREAD_START_ROUTINE lpStartAddress,
    a,      // _In_opt_   LPVOID lpParameter,
    0,      // _In_       DWORD dwCreationFlags,
    NULL    // _Out_opt_  LPDWORD lpThreadId
  );

  return NULL==a->hThread?NULL:a;
}

void audio_wait(audio_t *a)
{
  if (NULL!=a->hThread) {
    WaitForSingleObject(a->hThread,INFINITE);
    CloseHandle(a->hThread);
    a->hThread=NULL;
  }
}

void audio_stop(audio_t *a)
{
#if defined (SOX) // [
  if (NULL!=a->chain) {
    sox_delete_effects_chain(a->chain);
    a->chain=NULL;
  }
#endif // ]

  a->st=NULL;
}

void audio_kill(audio_t *a)
{
  samples_kill(&a->s);
}

#if defined (IN_FFSOX_BS1770) // {
void audio_adapt(audio_t *a)
{
  //DMARKLN();
#if defined (SOX) // [
  samples_adapt(&a->s,NULL==a->chain?&a->s.q:&a->ar.q);
#else // ] [
  samples_adapt(&a->s,&a->s.q);
#endif // ]
}
#endif // }

DWORD WINAPI audio_thread(LPVOID lpParam)
{
  audio_t *a=lpParam;
  ffsox_ar_t *ar=&a->ar;
#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
  ffi_st_t *ast=a->dec->t.ffi.ast;
#endif // ]
#if defined (LIBFFSOX_BS1770) // [
#if defined (IN_FFSOX_BS1770) // [
  bs1770_lufs_t *lufs=&a->s.lufs;
#else // ] [
  bs1770_lufs_t *lufs=NULL;
#endif // ]
#endif // ]
#if defined (SOX) // [
  int err;
#endif // ]

#if defined (FFSOX_DEPRECATED_STREAM_CODEC) // [
#if defined (LIBFFSOX_BS1770) // [
  ffsox_ar_reset(ar,ast,a->s.q,lufs);
#else // ] [
  ffsox_ar_reset(ar,ast,a->s.q);
#endif // ]
#else // ] [
#if defined (LIBFFSOX_BS1770) // [
  ffsox_ar_reset(ar,a->s.q,lufs);
#else // ] [
  ffsox_ar_reset(ar,a->s.q);
#endif // ]
#endif // ]

#if defined (SOX) // [
  if (NULL==a->chain) {
#endif // ]
    DWRITELN("NATIVE ...");
    ffsox_ar_next(ar);

    while (ffsox_ar_valid(ar)) {
      samples_copy(&a->s,ar);
      ffsox_ar_next(ar);
    }
#if defined (SOX) // [
  }
  else {
    DVWRITELN("SOX: chain=%p ...",a->chain);

    if (SOX_SUCCESS!=(err=sox_flow_effects(a->chain,NULL,NULL)))
      DVWRITELN("WARNING: flow SoX effects (%s)",sox_error_mnemonic(err));
  }
#endif // ]

  // stop samples
  WaitForSingleObject(a->s.hMutex,INFINITE);

  if (STATE_RUN==a->s.state)
    a->s.state=STATE_EOF;

  SetEvent(a->s.hEventRead);
  ReleaseMutex(a->s.hMutex);

  return 0;
}
